Package: visNetwork
Title: Network Visualization using 'vis.js' Library
Version: 2.1.4
Authors@R: c(
    person(family = "Almende B.V. and Contributors", role = c("aut", "cph"),
       comment = "vis.js library in htmlwidgets/lib, https://visjs.org/, https://github.com/visjs/vis-network"),
    person("Benoit", "Thieurmel", role = c("aut", "cre"),
       comment = "R interface", email = "bthieurmel@gmail.com")
    )
Maintainer: Benoit Thieurmel <bthieurmel@gmail.com>
Description: Provides an R interface to the 'vis.js' JavaScript charting
    library. It allows an interactive visualization of networks.
BugReports: https://github.com/datastorm-open/visNetwork/issues
URL: https://datastorm-open.github.io/visNetwork/
Depends: R (>= 3.0)
Imports: htmlwidgets, htmltools, jsonlite, magrittr, utils, methods,
        grDevices, stats
License: MIT + file LICENSE
Encoding: UTF-8
Suggests: knitr, rmarkdown, webshot, igraph, rpart, shiny,
        shinyWidgets, colourpicker, sparkline, ggraph, tidygraph,
        flashClust
VignetteBuilder: knitr, rmarkdown
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-09-04 09:23:19 UTC; BenoîtThieurmel
Author: Almende B.V. and Contributors [aut, cph] (vis.js library in
    htmlwidgets/lib, https://visjs.org/,
    https://github.com/visjs/vis-network),
  Benoit Thieurmel [aut, cre] (R interface)
Repository: CRAN
Date/Publication: 2025-09-04 10:10:02 UTC
Built: R 4.6.0; ; 2026-04-28 22:43:10 UTC; unix
