# Generated by roxygen2: do not edit by hand

S3method(visHclust,data.frame)
S3method(visHclust,default)
S3method(visHclust,dist)
S3method(visHclust,hclust)
export("%>%")
export(addExport)
export(addFontAwesome)
export(addIonicons)
export(renderVisNetwork)
export(toVisNetworkData)
export(visClusteringByColor)
export(visClusteringByConnection)
export(visClusteringByGroup)
export(visClusteringByHubsize)
export(visClusteringOutliers)
export(visCollapse)
export(visConfigure)
export(visDocumentation)
export(visEdges)
export(visEvents)
export(visExport)
export(visFit)
export(visFocus)
export(visGetBoundingBox)
export(visGetConnectedEdges)
export(visGetConnectedNodes)
export(visGetEdges)
export(visGetNodes)
export(visGetPositions)
export(visGetScale)
export(visGetSelectedEdges)
export(visGetSelectedNodes)
export(visGetSelection)
export(visGetViewPosition)
export(visGroups)
export(visHclust)
export(visHierarchicalLayout)
export(visIgraph)
export(visIgraphLayout)
export(visInteraction)
export(visLayout)
export(visLegend)
export(visMoveNode)
export(visNearestNodes)
export(visNetwork)
export(visNetworkEditor)
export(visNetworkEditorServer)
export(visNetworkEditorUI)
export(visNetworkOutput)
export(visNetworkProxy)
export(visNodes)
export(visOptions)
export(visPhysics)
export(visRedraw)
export(visRemoveEdges)
export(visRemoveNodes)
export(visSave)
export(visSelectEdges)
export(visSelectNodes)
export(visSetData)
export(visSetOptions)
export(visSetSelection)
export(visSetTitle)
export(visStabilize)
export(visStartSimulation)
export(visStopSimulation)
export(visStorePositions)
export(visTree)
export(visTreeEditor)
export(visTreeModuleServer)
export(visTreeModuleUI)
export(visUncollapse)
export(visUnselectAll)
export(visUpdateEdges)
export(visUpdateNodes)
import(htmltools)
import(htmlwidgets)
importFrom(grDevices,boxplot.stats)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRamp)
importFrom(grDevices,hcl)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(jsonlite,fromJSON)
importFrom(magrittr,"%>%")
importFrom(methods,findFunction)
importFrom(stats,as.formula)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,na.omit)
importFrom(utils,browseURL)
importFrom(utils,getAnywhere)
importFrom(utils,packageVersion)
