% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visGetUtils.R
\name{visStorePositions}
\alias{visStorePositions}
\title{Method storePositions, with shiny only.}
\usage{
visStorePositions(graph)
}
\arguments{
\item{graph}{: a \code{\link{visNetworkProxy}}  object}
}
\description{
Method storePositions, with shiny only. Put the X and Y positions of all nodes into that dataset.
}
\examples{
\dontrun{

# have a look to : 
shiny::runApp(system.file("shiny", package = "visNetwork"))

}

}
\references{
See online documentation \url{https://datastorm-open.github.io/visNetwork/}
}
\seealso{
\link{visNodes} for nodes options, \link{visEdges} for edges options, \link{visGroups} for groups options, 
\link{visLegend} for adding legend, \link{visOptions} for custom option, \link{visLayout} & \link{visHierarchicalLayout} for layout, 
\link{visPhysics} for control physics, \link{visInteraction} for interaction, \link{visNetworkProxy} & \link{visFocus} & \link{visFit} for animation within shiny,
\link{visDocumentation}, \link{visEvents}, \link{visConfigure} ...
}
